package ittp.protocol.request.headers;

import ittp.protocol.request.Request;
import ittp.protocol.Protocol;
import ittp.ittpException;
import ittp.protocol.response.Response;
import java.text.ParsePosition;
import java.text.FieldPosition;
import java.text.DateFormat;

/**
 * This class analyse the header connection send by the client.
 * @author Frederic Bidon and Mathieu Texier
 */
public class Date extends Request.Headers {
    private java.util.Date datetime;
    private boolean used = false;
    private Request parent;
    /**Constructor of the class*/
    public Date (Request parent) {
        parent.super ();
        this.parent = parent;
    }
    
    /**
     * Mutators for the String dateText.
     * This method convert the date to the dateFormat.
     * @param dateText String that contains the date when the request has been send.
     * @throws ittpException if the headers DateTime already received or if the format of the date is incorrect
     * or if the request is old than 10 minutes.
     */
    public void setDateTime (String dateText) throws ittpException {
        if (used)
            throw new ittpException ("headers DateTime already received", Response.Code.ITTP_BAD_REQUEST);
        ParsePosition parse = new ParsePosition (0);
        datetime = Protocol.dateFormat.parse (dateText, parse);
        int errorIndex = parse.getErrorIndex ();
        if (errorIndex != -1)
            throw new ittpException ("Wrong date Format " + dateText + " at character n=" + errorIndex, Response.Code.ITTP_BAD_REQUEST);
        
        java.util.Date now = new java.util.Date ();
        java.util.Date tenMinuteAgo = new java.util.Date (now.getTime ()- 600000);
        if (datetime.after (now) || datetime.before (tenMinuteAgo))
            throw new ittpException ("Date expired or wrong date", Response.Code.ITTP_NOT_FOUND);
        used = true;
    }
    
    /**
     * Accessors for the boolean used.
     * @return used: boolean that determine if this headers is used.
     */
    public String getDateTime () {
        StringBuffer strBuf = new StringBuffer ();
        strBuf = Protocol.dateFormat.format (datetime, strBuf ,new FieldPosition (0));
        return strBuf.toString ();
    }
    
    /**
     * Convert to a string the headers date send by the client.
     * @return a string containing the response to this headers if it is used.
     */
    public String toString () {
        if (used)
            return "Date:" + getDateTime () + "\n";
        else return "";
    }
    
    /**
     * Accessors for the boolean used.
     * @return used: boolean that determine if this headers is used.
     */
    public boolean isUsed () {
        return used;
    }
}
